/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.client.isobus.model;

import nl.cloudfarming.client.util.UOM;

/**
 *
 * @author johan
 */
public enum DDI {
    
    DDI_1("Setpoint Volume Per Area Application Rate", UOM.MM3_PER_M2, new byte[]{0, 1}, 0.01),
    DDI_6("Setpoint Mass Per Area Application Rate", UOM.MG_PER_M2, new byte[]{0, 6}),
    DDI_16("Setpoint Spacing Application Rate", UOM.MM, new byte[]{0, 16}),
    DDI_141("Actual Work State", null, new byte[]{0, Byte.parseByte("-115")}), // AKA on/off
    ;
    // TODO: add more DDI values...
    // latest version is available at http://dictionary.isobus.net/isobus/
    
    public static final double DEFAULT_RESOLUTION = 1.0;

    private final String description;
    private final byte[] DDEntityId; // length 2
    private final UOM uom; //todo: to enum; maybe later: generic data conversion based on input value and from/to UOMs.
    private final double resolution;

    private DDI(String description, UOM uom, byte[] DDEntityId, double resolution) {
        this.DDEntityId = DDEntityId;
        this.description = description;
        this.uom = uom;
        this.resolution = resolution;
    }
    
    private DDI(String description, UOM uom, byte[] DDEntityId) {
        this(description, uom, DDEntityId, DEFAULT_RESOLUTION);
    }

    public String getDescription() {
        return description;
    }

    public byte[] getDDEntityId() {
        return DDEntityId;
    }

    public UOM getUom() {
        return uom;
    }

    public double getResolution() {
        return resolution;
    }
    
    public int toExportValue(double value) {
        return (int)(value / resolution);
    }
    
    public double toImportValue(int value) {
        return (((double)value) * resolution);
    }
    
    @Override
    public String toString() {
        String result = description;
        if (uom != null) {
            result += ", " + uom.getLabel();
        }
        return result;
    }
    
}
