/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.model;

import nl.cloudfarming.client.task.ExportableTask;
import nl.cloudfarming.client.util.Callback;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;

/**
 * Manager to which handles isobus taskdata files
 * To register your implementation of this manager add ServiceProvider(Service=IsobusManager.class)
 * to your implementation
 * 
 * @author Merijn Zengers
 */
public interface IsobusManager {
    
   
    /**
     * Get all the taskdata folders from the provided project
     * These folders contain a TASKDATA.XML and optional GRD.BIN ,TLG.BIN and TLG.XML files 
     * @param project the project to search in
     * @return the found folder
     */
    public FileObject[] getTaskDataFolders(Project project);
    
    /**
     * Returns the folder where the Isobus files are stored
     * @param project the project where the isobus files belong
     * @return fileObject Isobus folder
     */
    public FileObject getIsobusFolder(Project project);
    
    /**
     * Export the given tasks to an isobus taskfile in the specified destination
     * folder.
     * 
     * 
     * @param destFolder destination folder
     * @param gridType 
     * @param ddi isobus data dictionary id
     * @param multiplier conversion factor to use when the TZN values use a different UOM than the DDI.
     * @param tasks Tasks to export
     */
    public void exportTaskdata(FileObject destFolder, GridTypeEnum gridType, DDI ddi, double multiplier, Callback<Object> callback, ExportableTask... tasks);
}
